﻿using System.Drawing;

using Svg;

namespace TestProject
{
    /// <summary>
    /// SVG 파서
    /// </summary>
    public class SVGParser
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 최대 크기 - MaximumSize

        /// <summary>
        /// 최대 크기
        /// </summary>
        public static Size MaximumSize { get; set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region SVG 문서 구하기 - GetSVGDocument(filePath)

        /// <summary>
        /// SVG 문서 구하기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        /// <returns>SVG 문서</returns>
        public static SvgDocument GetSVGDocument(string filePath)
        {
            SvgDocument document = SvgDocument.Open(filePath);

            return AdjustSize(document);
        }

        #endregion
        #region 비트맵 구하기 - GetBitmap(filePath)

        /// <summary>
        /// 비트맵 구하기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        /// <returns>비트맵</returns>
        public static Bitmap GetBitmap(string filePath)
        {
            SvgDocument document = GetSVGDocument(filePath);
         
            Bitmap bitmap = document.Draw();

            return bitmap;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 크기 조정하기 - AdjustSize(document)

        /// <summary>
        /// 크기 조정하기
        /// </summary>
        /// <param name="document">SVG 문서</param>
        /// <returns>SVG 문서</returns>
        private static SvgDocument AdjustSize(SvgDocument document)
        {
            if(document.Height > MaximumSize.Height)
            {
                document.Width  = (int)((document.Width / (double)document.Height) * MaximumSize.Height);
                document.Height = MaximumSize.Height;
            }

            return document;
        }

        #endregion
    }
}